#include <cstdint>
#include <cassert>
#include <cstdio>
#include <cstring>
#include <vector>
#include <functional>
#include <algorithm>

const int N = 5e4 + 10;

struct BitVector {
  using u32 = uint32_t;
  std::vector<u32> blocks, ranktable;
  int len, blocks_len, count;
  BitVector(int n = 0) { init(n); }
  void set(int i) { blocks[i >> 5] |= 1 << (i & 31); }
  bool get(int i) const { return blocks[i >> 5] >> (i & 31) & 1; }
  void init(int n) {
    len = n;
    blocks_len = (len + 31) / 32 + 1;
    blocks.assign(blocks_len, 0);
  }
  void build() {
    count = 0;
    if (len == 0) return;
    ranktable.assign(blocks_len + 1, 0);
    for (int i = 0; i < blocks_len; ++i) {
      ranktable[i] = count;
      count += popcount(blocks[i]);
    }
    ranktable[blocks_len] = count;
  }
  int rank(int p) const { // number of 1s in [0, p)
    int idx = p >> 5;
    return ranktable[idx] + popcount(blocks[idx] & (1u << (p & 31)) - 1);
  }
  int rank(bool b, int p) const { return b ? rank(p) : p - rank(p); }
  int rank(bool b, int l, int r) const { return rank(b, r) - rank(b, l); }

private:
  static inline int popcount(u32 x) {
    x = x - ((x >> 1) & 0x55555555);
    x = (x & 0x33333333) + ((x >> 2) & 0x33333333);
    return ((x + (x >> 4) & 0xF0F0F0F) * 0x1010101) >> 24;
  }
};

template <class T> struct WaveletMatrix {
  using value_t = T;
  int height, len;
  value_t maxval;
  std::vector<BitVector> B;
  std::vector<int> mids;
  void init(const value_t* data, int n) {
    len = n;
    maxval = len ? *std::max_element(data, data + n) : 0;
    for (height = 1; maxval >> 1 >> (height - 1); ++height);
    B.assign(height, BitVector(len));
    mids.assign(height, 0);
    std::vector<value_t> now(data, data + n), next(n);
    for (int i = 0; i < height; ++i) {
      for (int j = 0; j < len; ++j) {
        mids[i] += (now[j] >> (height - 1 - i) & 1) == 0;
      }
      BitVector &bv = B[i];
      int zero = 0, one = mids[i];
      for (int j = 0; j < len; ++j) {
        bool b = now[j] >> (height - 1 - i) & 1;
        if (b) next.at(one++) = now[j], bv.set(j);
        else next.at(zero++) = now[j];
      }
      bv.build();
      next.swap(now);
    }
  }
  value_t get(int p) const {
    value_t ret = 0;
    for (int i = 0; i < height; ++i) {
      const BitVector &bv = B[i];
      bool dir = bv.get(p);
      ret = ret << 1 | dir;
      p = bv.rank(dir, p);
      if (dir) p += mids[i];
    }
    return ret;
  }
  // k-th (k >= 0) largest element in position [left, right)
  value_t quantile(int left, int right, int k) const {
    if (k < 0 || right - left <= k) {
      return -1;
    }
    value_t val = 0;
    for (int i = 0; i < height; ++i) {
      const BitVector &bv = B[i];
      int count = bv.rank(1, left, right);
      bool dir = k < count;
      val = val << 1 | (dir ? 1 : 0);
      if (!dir) k -= count;
      left = bv.rank(dir, left), right = bv.rank(dir, right);
      if (dir) left += mids[i], right += mids[i];
    }
    return val;
  }
  // number of elements less than val in position [left, right)
  int rank(value_t val, int left, int right, int ret = 0) const {
    if (val > maxval) return right - left;
    for (int i = 0; i < height; ++i) {
      const BitVector &bv = B[i];
      bool dir = val >> (height - i - 1) & 1;
      int leftcount = bv.rank(dir, left), rightcount = bv.rank(dir, right);
      if (dir) ret += (right - left) - (rightcount - leftcount);
      left = leftcount, right = rightcount;
      if (dir)
        left += mids[i], right += mids[i];
    }
    return ret;
  }
  // smallest value >= v in position [left, right)
  value_t succ(int left, int right, value_t v) const {
    int lt = rank(v, left, right);
    if (lt == right - left) return -1;
    return quantile(left, right, right - left - lt - 1);
  }
  // largeest value <= v in position [left, right)
  value_t pred(int left, int right, value_t v) const {
    int le = rank(v + 1, left, right);
    if (le == 0) return -1;
    return quantile(left, right, right - left - le);
  }
};

struct SparseTable {
  int u[17][N], n;
  void build(int a[], int n) {
    this->n = n;
    if (n == 0) return;
    for (int i = 0; i < n; ++i) u[0][i] = a[i];
    for (int s = 1, m = 1; m * 2 <= n; ++s, m <<= 1) {
      for (int i = 0; i + m < n; ++i) {
        u[s][i] = std::min(u[s - 1][i], u[s - 1][i + m]);
      }
    }
  }
  // find the minimum value in range [l, r)
  int query(int l, int r) const {
    if (l == r) return 0;
    int k = 31 - __builtin_clz(r - l);
    return std::min(u[k][l], u[k][r - (1 << k)]);
  }
};

namespace sa {
int sa[N], rk[N], ht[N], s[N<<1], t[N<<1], p[N], cnt[N], cur[N];
SparseTable lcpRMQ;
#define pushS(x) sa[cur[s[x]]--] = x
#define pushL(x) sa[cur[s[x]]++] = x
#define inducedSort(v) std::fill_n(sa, n, -1); std::fill_n(cnt, m, 0);      \
  for (int i = 0; i < n; i++) cnt[s[i]]++;                                  \
  for (int i = 1; i < m; i++) cnt[i] += cnt[i-1];                           \
  for (int i = 0; i < m; i++) cur[i] = cnt[i]-1;                            \
  for (int i = n1-1; ~i; i--) pushS(v[i]);                                  \
  for (int i = 1; i < m; i++) cur[i] = cnt[i-1];                            \
  for (int i = 0; i < n; i++) if (sa[i] > 0 &&  t[sa[i]-1]) pushL(sa[i]-1); \
  for (int i = 0; i < m; i++) cur[i] = cnt[i]-1;                            \
  for (int i = n-1;  ~i; i--) if (sa[i] > 0 && !t[sa[i]-1]) pushS(sa[i]-1)
void sais(int n, int m, int *s, int *t, int *p) {
  int n1 = t[n-1] = 0, ch = rk[0] = -1, *s1 = s+n;
  for (int i = n-2; ~i; i--) t[i] = s[i] == s[i+1] ? t[i+1] : s[i] > s[i+1];
  for (int i = 1; i < n; i++) rk[i] = t[i-1] && !t[i] ? (p[n1] = i, n1++) : -1;
  inducedSort(p);
  for (int i = 0, x, y; i < n; i++) if (~(x = rk[sa[i]])) {
    if (ch < 1 || p[x+1] - p[x] != p[y+1] - p[y]) ch++;
    else for (int j = p[x], k = p[y]; j <= p[x+1]; j++, k++)
      if ((s[j]<<1|t[j]) != (s[k]<<1|t[k])) {ch++; break;}
    s1[y = x] = ch;
  }
  if (ch+1 < n1) sais(n1, ch+1, s1, t+n, p+n1);
  else for (int i = 0; i < n1; i++) sa[s1[i]] = i;
  for (int i = 0; i < n1; i++) s1[i] = p[sa[i]];
  inducedSort(s1);
}
template<typename T>
int mapCharToInt(int n, const T *str) {
  int m = *std::max_element(str, str+n);
  std::fill_n(rk, m+1, 0);
  for (int i = 0; i < n; i++) rk[str[i]] = 1;
  for (int i = 0; i < m; i++) rk[i+1] += rk[i];
  for (int i = 0; i < n; i++) s[i] = rk[str[i]] - 1;
  return rk[m];
}
template<typename T>
void build(const T *str, int n) {
  int m = mapCharToInt(++n, str);
  sais(n, m, s, t, p);
  for (int i = 0; i < n; i++) rk[sa[i]] = i;
  for (int i = 0, h = ht[0] = 0; i < n-1; i++) {
    int j = sa[rk[i]-1];
    while (i+h < n && j+h < n && s[i+h] == s[j+h]) h++;
    if ((ht[rk[i]] = h)) h--;
  }
  lcpRMQ.build(ht, n);
}
int rmq(int i, int j) {
  return lcpRMQ.query(i, j + 1);
}
int computeLCP(int i, int j) {
  if (i == j) return lcpRMQ.n - i;
  int x = rk[i], y = rk[j];
  if (x > y) std::swap(x, y);
  return rmq(x + 1, y);
}
}

// periodic progression
struct AP {
  int st, ed, d;
};

WaveletMatrix<int> wm;
char s[N];
int n, m;

std::pair<int, int> lcp_range(int p, int l) {
  int vl = 0, vr = 0, k = 0, s = 0;
  for (s = 1, k = 0; p >= s && sa::rmq(p - s + 1, p) >= l; ++k, s *= 2);
  for (; k >= 0; --k, s >>= 1) {
    if (p >= vl + s && sa::rmq(p - vl - s + 1, p) >= l) vl += s;
  }
  for (s = 1, k = 0; p + s <= n && sa::rmq(p + 1, p + s) >= l; ++k, s *= 2);
  for (; k >= 0; --k, s >>= 1) {
    if (p + vr + s <= n && sa::rmq(p + 1, p + vr + s) >= l) vr += s;
  }
  return {p - vl, p + vr};
}

// occ of s[xl, xr) in s[yl, yr) as single AP, yr - yl <= 2(xr - yl)
AP ipm_query(int xl, int xr, int yl, int yr) {
  if (xl == xr) return {-1, -1, -1};
  int len = xr - xl;
  assert(yr - yl <= len * 2);
  auto iv = lcp_range(sa::rk[xl], len);
  int st = wm.succ(iv.first, iv.second + 1, yl);
  // p + len <= yr
  if (st == -1 || st > yr - len) return {-1, -1, -1};
  int ed = wm.pred(iv.first, iv.second + 1, yr - len);
  if (st == ed) return {st, ed, 0};
  int ns = wm.succ(iv.first, iv.second + 1, st + 1);
  return {st, ed, ns - st};
}

int get_lcp(int xl, int xr, int yl, int yr) {
  return std::min(sa::computeLCP(xl, yl), std::min(xr - xl, yr - yl));
}

int cmp_str(int xl, int xr, int yl, int yr) {
  int l = get_lcp(xl, xr, yl, yr);
  int cx = (l == xr - xl) ? 0 : s[xl + l];
  int cy = (l == yr - yl) ? 0 : s[yl + l];
  return (cx < cy) ? -1 : (cx > cy);
}

namespace wst {
struct Node {
  int l, r; // range in suffix array
  int label, lcp;
  int ht;
  Node *ch[2]; // left/right children
} pool[N * 4], *ptr, *root;

void init() {
  ptr = pool;
}

Node* new_node(int l, int r, int label) {
  ptr->l = l, ptr->r = r;
  ptr->label = label;
  if (l + 1 == r) ptr->lcp = n - sa::sa[l] + 1;
  else {
    int xl = sa::sa[l], xr = xl + std::max(sa::ht[l] + 1, label / 2);
    int yl = sa::sa[r - 1], yr = n;
    ptr->lcp = get_lcp(xl, xr, yl, yr);;
  }
  ptr->ht = 1;
  ptr->ch[0] = ptr->ch[1] = 0;
  return ptr++;
}

Node* new_node(Node *ls, Node *rs, int label) {
  assert(ls->r == rs->l);
  ptr->l = ls->l, ptr->r = rs->r;
  ptr->ht = std::max(ls->ht, rs->ht) + 1;
  ptr->label = label;
  int xl = sa::sa[ptr->l], xr = xl + std::max(sa::ht[ptr->l] + 1, label / 2);
  int yl = sa::sa[ptr->r - 1], yr = n;
  ptr->lcp = get_lcp(xl, xr, yl, yr);;
  ptr->ch[0] = ls, ptr->ch[1] = rs;
  return ptr++;
}

std::vector<std::vector<int>> build_from_lca(const std::vector<int> &lca) {
  std::vector<std::vector<int>> edges(lca.size() * 2 + 1);
  std::vector<std::pair<int, int>> stk = {{0, 0}, {1, 1e9}};
  int idx = 2;
  for (auto &l: lca) {
    int last = -1;
    while (!stk.empty() && l < stk.back().second) {
      if (last != -1) {
        edges[stk.back().first].push_back(last);
      }
      last = stk.back().first;
      stk.pop_back();
    }
    assert(!stk.empty());
    if (l != stk.back().second) {
      edges[idx].push_back(last);
      stk.emplace_back(idx++, l);
    } else {
      edges[stk.back().first].push_back(last);
    }
    stk.emplace_back(idx++, 1e9);
  }
  for (int last = -1; !stk.empty(); stk.pop_back()) {
    if (last != -1) {
      edges[stk.back().first].push_back(last);
    }
    last = stk.back().first;
  }
  return edges;
}

Node* build_from_subtree(const std::vector<Node*> &nodes, int l, int r, int label) {
  if (l + 1 == r) return nodes[l];
  std::vector<int> lca(r - l);
  for (int i = l; i < r; ++i) {
    lca[i - l] = nodes[i]->r - nodes[i]->l;
    if (i != l) lca[i - l] += lca[i - l - 1];
  }
  for (int i = r - l - 1; i >= 0; --i) {
    int v = lca[i] ^ (i == 0 ? 0 : lca[i - 1]);
    lca[i] = 31 - __builtin_clz(v);
  }
  int w = *std::max_element(lca.begin(), lca.end());
  for (auto &x: lca) x = w - x;
  auto edges = build_from_lca(lca);

  std::function<Node*(int)> dfs = [&](int u) -> Node* {
    if (edges[u].size() == 0) return NULL;
    assert(edges[u].size() == 2);
    std::vector<Node*> ws;
    auto ls = dfs(edges[u][0]);
    if (ls) ws.push_back(ls);
    ws.push_back(nodes[l++]);
    auto rs = dfs(edges[u][1]);
    if (rs) ws.push_back(rs);
    if (ws.size() == 1) return ws[0];
    if (ws.size() == 2) return new_node(ws[0], ws[1], label);
    return new_node(new_node(ws[0], ws[1], label), ws[2], label);
  };

  return dfs(0);
}

// build wavelet suffix tree in linear time
void build(int n, char *s) {
  sa::build(s, n);
  wm.init(sa::sa, n + 1);
  std::vector<Node*> nodes(n + 1);
  for (int i = 0; i <= n; ++i) {
    nodes[i] = new_node(i, i + 1, (n - sa::sa[i] + 1) * 2);
  }
  int w = 1;
  while (w * 2 <= n) w *= 2;
  while (w >= 1) {
    std::vector<Node*> tmp;
    for (int i = 0, j, k = 0; i <= n; i = j) {
      for (j = i; j <= n && (i == j || sa::rmq(i + 1, j) >= w); ++j);
      int vl = k;
      while (k < (int)nodes.size() && nodes[k]->r <= j) ++k;
      tmp.push_back(build_from_subtree(nodes, vl, k, w * 2));
    }
    nodes.swap(tmp);
    w >>= 1;
  }
  root = build_from_subtree(nodes, 0, nodes.size(), 1);
}

// number of suffix s[p..r) where p in AP such that
// s[xl..xr) <= s[p..r) <= s[yl..yr) in O(log n)
int count_ap(AP ap, int r, int xl, int xr, int yl, int yr) {
  if (ap.st == -1) return 0;
  if (ap.st == ap.ed) return cmp_str(ap.st, r, xl, xr) >= 0 && cmp_str(ap.st, r, yl, yr) <= 0;
  if (cmp_str(ap.st, r, ap.st + ap.d, r) == 1) {
    std::swap(ap.st, ap.ed), ap.d *= -1;
  }
  if (cmp_str(ap.ed, r, xl, xr) < 0) return 0;
  if (cmp_str(ap.st, r, yl, yr) > 0) return 0;
  int sx = 0, sy = 0, k = 0, sk = ap.d, cnt = (ap.ed - ap.st) / ap.d;
  for (; (1 << k) <= cnt && cmp_str(ap.st + sk, r, xl, xr) < 0; sk *= 2, ++k);
  for (; k >= 0; --k, sk >>= 1) {
    if (sx + (1 << k) <= cnt && cmp_str(ap.st + ap.d * sx + sk, r, xl, xr) < 0) sx += 1 << k;
  }
  if (cmp_str(ap.st + sx * ap.d, r, xl, xr) >= 0) --sx;
  k = 0, sk = ap.d;
  for (; (1 << k) <= cnt && cmp_str(ap.st + sk, r, yl, yr) <= 0; sk *= 2, ++k);
  for (; k >= 0; --k, sk >>= 1) {
    if (sy + (1 << k) <= cnt && cmp_str(ap.st + ap.d * sy + sk, r, yl, yr) <= 0) sy += 1 << k;
  }
  return sy - sx;
}

// number of suffixes of s[l..r) in edge (p, q) <= s[k..r) in O(log n)
int count_edge(int l, int r, int k, Node *p, Node *q) {
  if (q->label == p->label) return 0;
  int lcp = sa::ht[q->l], st = sa::sa[q->l];
  int edl = std::max(lcp + 1, p->label / 2);
  int edr = q->label / 2 - 1;
  if (edl > edr) return 0;
  int m = std::max(l, r - p->label + 1), ret = 0;
  // solve suffixes of s[m..r)
  auto ap = ipm_query(st, st + edl, m, r);
  if (cmp_str(st, st + edr, k, r) <= 0) {
    ret += count_ap(ap, r, st, st + edl, st, st + edr);
  } else {
    ret += count_ap(ap, r, st, st + edl, k, r);
  }
  if (l == m) return ret;
  l = std::max(l, r - q->label + 1);
  int bound = std::min(m + q->lcp - 1, n + 1);
  // solve suffix s[x..r) where x in [l, m)
  ap = ipm_query(st, st + q->lcp, l, std::min(l + q->lcp * 2, bound));
  if (cmp_str(st, st + edr, k, r) <= 0) {
    ret += count_ap(ap, r, st, st + edl, st, st + edr);
  } else {
    ret += count_ap(ap, r, st, st + edl, k, r);
  }
  if (q->lcp + 1 >= m) return ret;
  ap = ipm_query(st, st + q->lcp, l + q->lcp + 1, bound);
  if (cmp_str(st, st + edr, k, r) <= 0) {
    ret += count_ap(ap, r, st, st + edl, st, st + edr);
  } else {
    ret += count_ap(ap, r, st, st + edl, k, r);
  }
  return ret;
}

// number of suffixed of s[l..r) in subtree p <= s[k..r) in O(log n)
int count_subtree(int l, int r, int k, Node *p) {
  int m = std::max(l, r - p->label + 1), ret = 0;
  // solve suffixes of s[m..r)
  auto ap = ipm_query(sa::sa[p->l], sa::sa[p->l] + p->lcp, m, r);
  int xl = sa::sa[p->l], xr = xl + std::max(sa::ht[p->l] + 1, p->label / 2);
  int yl = sa::sa[p->r - 1], yr = n;
  ret += count_ap(ap, r, xl, xr, yl, yr);
  // solve suffix s[x..r) where x in [l, m)
  return ret + wm.rank(m, p->l, p->r) - wm.rank(l, p->l, p->r);
}

// substring suffix rank of s[l..r) in O(log^2 n)
int rank(int l, int r, int k) {
  auto tar = lcp_range(sa::rk[k], r - k).first;
  Node *p = root;
  int ret = 0;
  while (p->ch[0] && p->ch[1] && p->label / 2 <= r - k) {
    ret += count_edge(l, r, k, p, p->ch[0]);
    if (p->ch[0]->l <= tar && tar < p->ch[0]->r) {
      p = p->ch[0];
    } else {
      ret += count_subtree(l, r, k, p->ch[0]);
      ret += count_edge(l, r, k, p, p->ch[1]);
      p = p->ch[1];
    }
  }
  return ret;
}
}

int main() {
  int T;
  scanf("%d", &T);
  for (int cas = 1; cas <= T; ++cas) {
    scanf("%d%d%s", &n, &m, s);
    wst::init();
    wst::build(n, s);
    for (int i = 0; i < m; ++i) {
      int l, r, k;
      scanf("%d%d%d", &l, &r, &k);
      printf("%d\n", wst::rank(l - 1, r, k - 1));
    }
  }
  return 0;
}
